﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;
using ZXing;

namespace LoadFromGallerySample
{
    public partial class MainPage : ContentPage
    {
        bool isOpenGallery = false;
        ImageModel model;

        Stream imageStream = new MemoryStream();

        public MainPage()
        {
            InitializeComponent();           
        }

        /// <summary>
        /// Upload the image from gallery    
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Button_OnClicked(object sender, EventArgs e)
        {         
            DependencyService.Get<ILoad>().UploadFromGallery(this);
        }


        /// <summary>
        /// Get the picked image as stream
        /// </summary>
        /// <param name="image"></param>
        public void SwitchView(Stream image)
        {
            imageStream = image;           

            var aditionalHints = new KeyValuePair<DecodeHintType, object>(key: DecodeHintType.PURE_BARCODE, value: "TRUE");
            var result = Decode(imageStream, file: "image_to_read", format: BarcodeFormat.QR_CODE, aditionalHints: new[] { aditionalHints });

            if (result != null)
            {
                //Scanned barcode value 
                string barcodeText = result.Text;
            }
        }
        /// <summary>
        /// Get RGB value from image stream and read the barcode text 
        /// </summary>
        /// <param name="imageStream"></param>
        /// <param name="file"></param>
        /// <param name="format"></param>
        /// <param name="aditionalHints"></param>
        /// <returns></returns>
        private Result Decode(Stream imageStream, string file, BarcodeFormat? format = null, KeyValuePair<DecodeHintType, object>[] aditionalHints = null)
        {
            var r = GetReader(format, aditionalHints);

            MemoryStream ms = new MemoryStream();
            imageStream.CopyTo(ms);
            var bytes = ms.ToArray();

            var binaryBitmap = DependencyService.Get<ILoad>().GetBinaryBitmap(bytes);

            var result = r.decode(binaryBitmap);

            return result;
        }

        /// <summary>
        /// Get zxing reader
        /// </summary>
        /// <param name="format"></param>
        /// <param name="aditionalHints"></param>
        /// <returns></returns>
        MultiFormatReader GetReader(BarcodeFormat? format, KeyValuePair<DecodeHintType, object>[] aditionalHints)
        {
            var reader = new MultiFormatReader();

            var hints = new Dictionary<DecodeHintType, object>();

            if (format.HasValue)
            {
                hints.Add(DecodeHintType.POSSIBLE_FORMATS, new List<BarcodeFormat>() { format.Value });
            }
            if (aditionalHints != null)
            {
                foreach (var ah in aditionalHints)
                {
                    hints.Add(ah.Key, ah.Value);
                }
            }

            reader.Hints = hints;

            return reader;
        }
    }
    class ImageModel
    {
        public ImageSource TakePic { get; set; }
        public ImageSource TakePicSelected { get; set; }
        public ImageSource ChooseImage { get; set; }
        public ImageSource ChooseImageSelected { get; set; }
        public ImageSource BroweImage1 { get; set; }
        public ImageSource BroweImage2 { get; set; }
        public ImageSource BroweImage3 { get; set; }

        public ImageModel()
        {
            ChooseImage = ImageSource.FromResource("IECameraandGallery.Icons.Gallery_S.png", typeof(App).GetTypeInfo().Assembly);//GallerySelected
            TakePic = ImageSource.FromResource("IECameraandGallery.Icons.Take_Photo_W.png", typeof(App).GetTypeInfo().Assembly);
            ChooseImageSelected = ImageSource.FromResource("IECameraandGallery.Icons.Gallery_W.png", typeof(App).GetTypeInfo().Assembly);
            TakePicSelected = ImageSource.FromResource("IECameraandGallery.Icons.Take_Photo_S.png", typeof(App).GetTypeInfo().Assembly);
            BroweImage1 = ImageSource.FromResource("IECameraandGallery.Icons.image2.png", typeof(App).GetTypeInfo().Assembly);
            BroweImage2 = ImageSource.FromResource("IECameraandGallery.Icons.image3.png", typeof(App).GetTypeInfo().Assembly);
            BroweImage3 = ImageSource.FromResource("IECameraandGallery.Icons.image4.png", typeof(App).GetTypeInfo().Assembly);
        }
    }
}
